<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Email\Models\EventEmail;
use EventPortal\Domain\Email\PreviewEmail;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Permission\Models\Permission;
use EventPortal\Http\Middleware\CurrentTeam;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Support\Facades\Mail;
use Spatie\Snapshots\MatchesSnapshots;

class EmailPreviewTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots;

    /** @test **/
    public function can_preview_an_email()
    {
        $admin = factory(Admin::class)->create();
        $event = factory(Event::class)->create([
            'name' => 'Hello World!',
        ]);
        $email = factory(EventEmail::class)->create([
            'event_id' => $event,
        ]);

        $admin->givePermissionTo([
            factory(Permission::class)->create([
                'name' => 'email.show',
            ]),
        ]);

        $this->withoutMiddleware(CurrentTeam::class)
            ->actingAs($admin->user)
            ->get(route('admin.email.preview.show', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();
    }

    /** @test **/
    public function can_send_an_email_preview_to_a_single_recipient()
    {
        Mail::fake();

        $admin = factory(Admin::class)->create();
        $event = factory(Event::class)->create([
            'name' => 'Hello World!',
        ]);
        $email = factory(EventEmail::class)->create([
            'event_id' => $event,
        ]);

        $admin->givePermissionTo([
            factory(Permission::class)->create([
                'name' => 'email.show',
            ]),
            factory(Permission::class)->create([
                'name' => 'email.update',
            ]),
        ]);

        $this->withoutMiddleware(CurrentTeam::class)
            ->actingAs($admin->user)
            ->followingRedirects()
            ->post(route('admin.email.preview.store', [
                'event' => $event,
                'email' => $email,
            ]), [
                'recipients' => 'test1@example.com',
            ])
            ->assertOk();

        Mail::assertSent(PreviewEmail::class, 1);

        Mail::assertSent(PreviewEmail::class, function ($email) {
            return $email->hasTo('test1@example.com');
        });
    }

    /** @test **/
    public function can_send_an_email_preview_to_multiple_recipients()
    {
        Mail::fake();

        $admin = factory(Admin::class)->create();
        $event = factory(Event::class)->create([
            'name' => 'Hello World!',
        ]);
        $email = factory(EventEmail::class)->create([
            'event_id' => $event,
        ]);

        $admin->givePermissionTo([
            factory(Permission::class)->create([
                'name' => 'email.show',
            ]),
            factory(Permission::class)->create([
                'name' => 'email.update',
            ]),
        ]);

        $this->withoutMiddleware(CurrentTeam::class)
            ->actingAs($admin->user)
            ->followingRedirects()
            ->post(route('admin.email.preview.store', [
                'event' => $event,
                'email' => $email,
            ]), [
                'recipients' => 'test1@example.com,test2@example.com',
            ])
            ->assertOk();

        Mail::assertSent(PreviewEmail::class, 2);

        Mail::assertSent(PreviewEmail::class, function ($email) {
            return $email->hasTo([
                'test1@example.com',
                'test2@example.com'
            ]);
        });
    }

    /** @test **/
    public function cannot_send_a_preview_email_to_an_invalid_address()
    {
        Mail::fake();

        $admin = factory(Admin::class)->create();
        $event = factory(Event::class)->create([
            'name' => 'Hello World!',
        ]);
        $email = factory(EventEmail::class)->create([
            'event_id' => $event,
        ]);

        $admin->givePermissionTo([
            factory(Permission::class)->create([
                'name' => 'email.show',
            ]),
            factory(Permission::class)->create([
                'name' => 'email.update',
            ]),
        ]);

        $response = $this->withoutMiddleware(CurrentTeam::class)
            ->actingAs($admin->user)
            ->post(route('admin.email.preview.store', [
                'event' => $event,
                'email' => $email,
            ]), [
                'recipients' => 'valid-email-address@example.com,invalid-email-address',
            ]);

        $response->assertSessionHasErrors([
            'recipients.*' => [
                'Please ensure invalid-email-address is a valid email address.'
            ],
        ]);

        Mail::assertNotSent(PreviewEmail::class);
    }

    /** @test **/
    public function the_email_preview_shows_a_preview_recipient()
    {
        $admin = factory(Admin::class)->create();
        $event = factory(Event::class)->create();
        $email = factory(EventEmail::class)->create([
            'event_id' => $event,
        ]);

        $admin->givePermissionTo([
            factory(Permission::class)->create([
                'name' => 'email.show',
            ]),
        ]);

        $response = $this->withoutMiddleware(CurrentTeam::class)
            ->actingAs($admin->user)
            ->get(route('admin.email.preview.show', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        $response->assertSee("Hi [Firstname]");
        $response->assertSee("This email was sent to test-address@example.com");
    }
}
